#! /usr/bin/python
"""
Use, distribution, and modification of the ScummPacker binaries, source code,
or documentation, is subject to the terms of the MIT license, as below.

Copyright (c) 2011 Laurence Dougal Myers

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import ConfigParser
import logging
from optparse import OptionParser
import os
import sys

import musicsort_analysis
import musicsort_namer
import musicsort_sorter
from musicsort_util import action_prompt, get_all_subdirs, get_all_files
from musicsort_util import AppError, NotFoundError, ArgumentError, AlreadyExistsError

__author__="Laurence Dougal Myers"
__date__ ="$03/06/2011 1:14:09 PM$"

# These values are read from an external configuration file.
MUSIC_ROOT = None
ALWAYS_DELETE_EMPTY_DIRS = None

# These are used for actually parsing the external configuration file.
CONFIG_FILENAME = u'musicsort.ini'
CONFIG_SECTION_MAIN = 'MusicSort'
CONFIG_OPTION_MUSIC_ROOT = 'music_root'
CONFIG_OPTION_ALWAYS_DELETE_EMPTY_DIRS = "delete_empty_dirs"

def sort_some_music_yall(dir_to_sort):
    global MUSIC_ROOT
    print u"%s" % dir_to_sort
    print u"Sorting..."
    try:
        album_data, manual_values_required = musicsort_analysis.analyze_dir(dir_to_sort)
    except NotFoundError, nfe:
        # If there are any sub-directories, we automatically explore them if
        #  there are no files in the parent directory, and prompt to explore them
        #  if files are present in the parent directory.
        if len(get_all_subdirs(dir_to_sort)) > 0:
            if len(get_all_files(dir_to_sort)) == 0:
                print u"No files found, exploring sub-dirs in %s." % os.path.split(dir_to_sort)[1]
                sort_subdirs(dir_to_sort)
            elif action_prompt(u"No music files found in %s. Process sub-dirs?" % os.path.split(dir_to_sort)[1]):
                sort_subdirs(dir_to_sort)
        return
    out_dir_name = musicsort_namer.get_output_name(album_data, manual_values_required)

    if out_dir_name is None:
        return
    destination = musicsort_sorter.get_destination(MUSIC_ROOT, out_dir_name)
    if destination is None:
        return
    print u"Moving to %s..." % destination
    musicsort_sorter.move_to_destination(dir_to_sort, destination)
    print u"Done!"

def sort_subdirs(root_dir):
    for d in get_all_subdirs(root_dir):
        try:
            sort_some_music_yall(d)
            if os.path.exists(d) and len(os.listdir(d)) == 0:
                if ALWAYS_DELETE_EMPTY_DIRS or action_prompt(u"No files or sub-dirs found in %s. Delete folder?" % os.path.split(d)[1]):
                    os.rmdir(d) # could throw some OS exceptions
            print "---"
        except Exception, e:
            # NOTE: this will prompt multiples times for the one exception
            #  if we're traversing any nested sub-directories.
            logging.exception(e)
            if action_prompt(u"Error encountered, proceed?"):
                continue
            else:
                raise e

def configure_logging():
    logging.basicConfig(format="", level=logging.INFO)

def validate_args(args, options):
    if len(args) != 2:
        logging.error(u"Must pass the dir to sort as an argument.")
        return False
    dir_to_sort = args[1]
    if not os.path.isdir(dir_to_sort):
        if os.path.isdir(os.path.join(os.getcwd(), dir_to_sort)):
            args[1] = os.path.isdir(os.path.join(os.getcwd(), dir_to_sort))
        else:
            raise ArgumentError(u"Could not find dir to sort: %s" % dir_to_sort)
    return True

def main(args):
    global MUSIC_ROOT
    global ALWAYS_DELETE_EMPTY_DIRS
    configure_logging()

    # Parse configuration items
    # (could be improved with handling exceptions, error scenarios etc)
    config = ConfigParser.ConfigParser()
    print os.path.join(os.path.split(args[0])[0], CONFIG_FILENAME)
    config.read(os.path.join(os.path.split(args[0])[0], CONFIG_FILENAME))
    MUSIC_ROOT = config.get(CONFIG_SECTION_MAIN, CONFIG_OPTION_MUSIC_ROOT)
    ALWAYS_DELETE_EMPTY_DIRS = config.getboolean(CONFIG_SECTION_MAIN, CONFIG_OPTION_ALWAYS_DELETE_EMPTY_DIRS)

    # Parse options & arguments
    oparser = OptionParser(usage="%prog [options] dir_to_sort",
                      version="1.0")

    oparser.add_option("-s", "--sub-dirs", action="store_true",
                      dest="do_sub_dirs", default=False,
                      help="Process all top level sub-dirs of the input directory.")

    options, args = oparser.parse_args(args)

    try:
        if not validate_args(args, options):
            oparser.print_help()
            return 1
    except ArgumentError, ae:
        logging.error(ae)
        return 1

    try:
        # Convert argument to Unicode. This prevents problems further down
        #  the line, when mutagen tries to read a file containing Unicode
        #  characters.
        dir_to_sort = args[1].decode(sys.stdout.encoding)
        if options.do_sub_dirs:
            sort_subdirs(dir_to_sort)
        else:
            sort_some_music_yall(dir_to_sort)
    except AlreadyExistsError, aee:
        logging.exception(u"Destination file already exists: \n")
        return 6
    except NotFoundError, nfe:
        logging.exception(u"File not found: \n")
        return 5
    except AppError, appe:
        logging.exception(u"Generic application exception: \n")
        return 4
    except Exception, _:
        logging.exception(u"Unhandled exception encountered: \n")
        return 3
    except KeyboardInterrupt, ki:
        logging.error("Program interrupted by user input (CTRL-C), quitting.")
        return 2

    return 0

if __name__ == "__main__":
    sys.exit(main(sys.argv))
