import os.path
import logging
from mutagen.easyid3 import EasyID3
from mutagen.mp3 import MP3
from mutagen.musepack import Musepack
from mutagen.oggvorbis import OggVorbis
from mutagen.easymp4 import EasyMP4
from mutagen.asf import ASF # WMA
import os
from musicsort_util import NotFoundError, NotSupportedError
from musicsort_util import extract_year_from_text_date, get_file_ext

SUPPORTED_MUSIC_FORMATS = frozenset((
    'MP3',
    'OGG',
    'AAC',
    'MP4',
    'M4A',
    'MPC',
    'WMA',
    'ASF',
    #'FLAC',
    #'APE',
    #'WV',
))

def import_audio_data(music_file_path, music_type):
    if music_type == "MP3":
        audio = MP3(music_file_path, ID3=EasyID3)
    elif music_type == "MPC":
        audio = Musepack(music_file_path) # untested
    elif music_type == "OGG":
        audio = OggVorbis(music_file_path) # untested
    elif music_type == "AAC" or music_type == "MP4" or music_type == "M4A":
        # Mutagen throws an error saying it can't find the key "moov" on some files.
        # Perhaps it only supports iTunes-compaitble M4A files.
        audio = EasyMP4(music_file_path)
    elif music_type == "WMA" or music_type == "ASF":
        audio = ASF(music_file_path)
    else:
        raise NotSupportedError("Unsupported music type: %s" % music_type)

    bitrate = audio.info.bitrate
    # Mutagen throws KeyError trying to look up "TDRC" on my test album without
    #  a year value.
    try:
        album = audio.tags['album'][0]
    except Exception, e:
        album = None
    try:
        year = audio.tags['date'][0]
        # We only want the year from the date field, so try some regex matching.
        year = extract_year_from_text_date(year)
    except Exception, e:
        year = None
    try:
        artist = audio.tags['artist'][0]
    except Exception, e:
        artist = None
    mtype = music_type

    return dict(bitrate=bitrate, album=album, year=year, artist=artist, mtype=mtype)

def get_all_music_files(in_path):
    global SUPPORTED_MUSIC_FORMATS
    out_list = []
    for in_file in os.listdir(in_path):
        file_type = get_file_ext(in_file)
        if file_type in SUPPORTED_MUSIC_FORMATS:
            out_list.append((os.path.join(in_path, in_file), in_file, file_type))
    return out_list

def fill_and_validate_album_data(album_data, audio_data, manual_values_required):
    values_to_check = ['artist', 'year', 'album']

    for vtc in values_to_check:
        if audio_data[vtc] is not None and audio_data[vtc] not in album_data[vtc]:
            album_data[vtc].append(audio_data[vtc])
        if len(album_data[vtc]) != 1:
            manual_values_required[vtc] = True

    album_data['bitrate'].append(audio_data['bitrate'])
    album_data['mtype'].append(audio_data['mtype'])


def analyze_dir(in_path):
    logging.debug("Analysing path: %s" % in_path)
    all_music_files = get_all_music_files(in_path)
    if len(all_music_files) == 0:
        logging.debug("No music files found in %s." % in_path)
        raise NotFoundError("No music files found in %s." % in_path)

    album_data = {
        'bitrate' : [],
        'album' : [],
        'year' : [],
        'artist' : [],
        'mtype' : [],
        'aux' : None # can only be specified in manual override
    }

    manual_values_required = {
        'album' : False,
        'year' : False,
        'artist' : False
    }

    for music_file_path, music_file, music_type in all_music_files:
        logging.debug("Analysing file: %s" % music_file)
        audio_data = import_audio_data(music_file_path, music_type)
        fill_and_validate_album_data(album_data, audio_data, manual_values_required)

    return album_data, manual_values_required

